<?php

namespace Drupal\media_remote_embed\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Path\PathValidatorInterface;
use Drupal\link\Plugin\Field\FieldFormatter\LinkFormatter;
use Drupal\media\Entity\MediaType;
use Drupal\media_remote_embed\Plugin\media\Source\RemoteEmbed;
use Drupal\media_remote_embed\Provider\ProviderFactory;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'remote_embed' formatter.
 *
 * @FieldFormatter(
 *   id = "remote_embed",
 *   label = @Translation("Remote Embed content"),
 *   field_types = {
 *     "link",
 *   },
 * )
 */
class RemoteEmbedFormatter extends LinkFormatter {

  /**
   * The RemoteEmbed Provider Factory.
   *
   * @var \Drupal\media_remote_embed\Provider\ProviderFactory
   */
  private ProviderFactory $providerFactory;

  /**
   * LoggerChannelFactoryInterface.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $logger;

  /**
   * Constructs a new LinkFormatter.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param string $label
   *   The formatter label display setting.
   * @param string $view_mode
   *   The view mode.
   * @param array $third_party_settings
   *   Third party settings.
   * @param \Drupal\Core\Path\PathValidatorInterface $path_validator
   *   The path validator service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory service.
   * @param \Drupal\media_remote_embed\Provider\ProviderFactory $provider_factory
   *   The RemoteEmbed Provider factory.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, $label, $view_mode, array $third_party_settings, PathValidatorInterface $path_validator, LoggerChannelFactoryInterface $logger_factory, ProviderFactory $provider_factory) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $label, $view_mode, $third_party_settings, $path_validator);

    $this->logger = $logger_factory->get('media_remote_embed');
    $this->providerFactory = $provider_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['label'],
      $configuration['view_mode'],
      $configuration['third_party_settings'],
      $container->get('path.validator'),
      $container->get('logger.factory'),
      $container->get('media_remote_embed.provider_factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
        'width' => 0,
        'height' => 0,
      ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    // Initially set the items using the LinkFormatter.
    $element = parent::viewElements($items, $langcode);

    $providers = $this->providerFactory->getAll();

    $width = $this->getSetting('width');
    $height = $this->getSetting('height');

    // If we find a matching provider for a url then override the element
    // with the providers embed code.
    foreach ($items as $delta => $item) {
      $uri = $item->uri;
      $matched_provider = NULL;

      foreach ($providers as $provider) {
        if (!$provider->isApplicable($uri)) {
          continue;
        }
        $matched_provider = $provider;
        break;
      }

      if (empty($matched_provider)) {
        $this->logger->error('Unable to locate a provider for the embed uri: @uri', ['@uri' => $uri]);
        continue;
      }

      $settings = [
        'uri' => $uri,
        'width' => $width,
        'height' => $height,
      ];

      $embed_render_array = $matched_provider->getEmbedMarkup($settings);

      if (empty($embed_render_array)) {
        $this->logger->error('No embed markup returned for embed uri: @uri from provider: @provider', [
          '@uri' => $uri,
          'provider' => get_class($matched_provider),
        ]);
        continue;
      }

      $element[$delta] = $embed_render_array;
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    return parent::settingsForm($form, $form_state) + [
      'width' => [
        '#type' => 'number',
        '#title' => $this->t('Width'),
        '#default_value' => $this->getSetting('width'),
        '#size' => 5,
        '#maxlength' => 5,
        '#field_suffix' => $this->t('pixels'),
        '#min' => 0,
        '#description' => t('Width of embed.'),
      ],
      'height' => [
        '#type' => 'number',
        '#title' => $this->t('Height'),
        '#default_value' => $this->getSetting('height'),
        '#size' => 5,
        '#maxlength' => 5,
        '#field_suffix' => $this->t('pixels'),
        '#min' => 0,
        '#description' => t('Height of embed.'),
      ],
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();
    if ($this->getSetting('width') && $this->getSetting('height')) {
      $summary[] = $this->t('Size: %width x %height pixels', [
        '%width' => $this->getSetting('max_width'),
        '%height' => $this->getSetting('max_height'),
      ]);
    }
    elseif ($this->getSetting('width')) {
      $summary[] = $this->t('Width: %width pixels', [
        '%width' => $this->getSetting('width'),
      ]);
    }
    elseif ($this->getSetting('height')) {
      $summary[] = $this->t('Height: %height pixels', [
        '%height' => $this->getSetting('height'),
      ]);
    }
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public static function isApplicable(FieldDefinitionInterface $field_definition) {
    if ($field_definition->getTargetEntityTypeId() !== 'media') {
      return FALSE;
    }

    if (parent::isApplicable($field_definition)) {
      $media_type = $field_definition->getTargetBundle();

      if ($media_type) {
        $media_type = MediaType::load($media_type);
        return $media_type && $media_type->getSource() instanceof RemoteEmbed;
      }
    }
    return FALSE;
  }

}
