<?php

namespace Drupal\media_remote_embed\Provider;

use Drupal\Component\Plugin\Exception\PluginException;

/**
 * The RemoteEmbed Provider Factory.
 */
class ProviderFactory {

  /**
   * The RemoteEmbed Provider plugin manager.
   *
   * @var \Drupal\Component\Plugin\PluginManagerInterface
   */
  protected $pluginManager;

  /**
   * ProviderFactory constructor.
   *
   * @param \Drupal\media_remote_embed\Provider\ProviderPluginManager $plugin_manager
   *   The RemoteEmbed Provider plugin manager.
   */
  public function __construct(ProviderPluginManager $plugin_manager) {
    $this->pluginManager = $plugin_manager;
  }

  /**
   * Returns the specified RemoteEmbed Provider.
   *
   * @param string $id
   *   The provider machine name.
   *
   * @return \Drupal\media_remote_embed\Provider\ProviderInterface|bool
   *   An RemoteEmbed provider instance.
   */
  public function get(string $id): ProviderInterface {
    try {
      /** @var \Drupal\media_remote_embed\Provider\ProviderInterface $provider */
      $provider = $this->pluginManager->createInstance($id);
      return $provider;
    }
    catch (PluginException $e) {
      watchdog_exception('remote_embed', $e);
    }
    return FALSE;
  }

  /**
   * Returns a list of providers.
   *
   * @return string[]
   *   An associative array of plugin labels keyed by plugin ID.
   */
  public function getList(): array {
    $list = [];
    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      $list[$id] = $definition['label'] ?? $id;
    }
    asort($list);
    return $list;
  }

  /**
   * Returns a loaded instance for each provider.
   *
   * @return array
   *   Contains an instance of each provider.
   */
  public function getAll(): array {
    $providers = [];
    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      try {
        /** @var \Drupal\media_remote_embed\Provider\ProviderInterface $provider */
        $providers[] = $this->pluginManager->createInstance($id);
      }
      catch (PluginException $e) {
        watchdog_exception('remote_embed', $e);
      }
    }
    return $providers;
  }

}
